bl_info = {
    "name": "MikuMikuRig", #插件名字
    "author": "William", #作者名字
    "version": (0, 4, 1,), #插件版本
    "blender": (2, 80, 0), #需要的*最低* blender 版本
    "location": "3DView > Tools", #插件所在位置
    "description": "自动为MMD模型生成rigify控制器", #描述
    "support": 'COMMUNITY', #支持等级（社区支持）
    "category": "Rigging", #分类
}
import bpy
import bpy_extras
from bpy.types import Operator
from . import translation
from . import operators
from bpy.props import BoolProperty
from bpy.props import IntProperty
from bpy.props import FloatProperty
from bpy.props import EnumProperty

def get_preset_item(self,context):
    preset_items=[('None','None','')]
    for name in operators.preset.preset_name_list:
        preset_items.append((name,name,''))
    return(preset_items)

class MMR_property(bpy.types.PropertyGroup):
    upper_body_controller:BoolProperty(default=True,description="上半身控制器")
    wrist_rotation_follow:BoolProperty(default=False,description="手腕旋转跟随手臂")
    auto_shoulder:BoolProperty(default=False,description="肩膀联动")
    solid_rig:BoolProperty(default=False,description="实心控制器")
    pole_target:BoolProperty(default=False,description="极向目标")
    min_ik_loop:IntProperty(default=10,description="最小IK迭代次数",min=1)
    lock_location:BoolProperty(default=False,description="锁定动画位置")
    fade_in_out:IntProperty(default=0,description="淡入淡出长度",min=0)
    action_scale:FloatProperty(default=1,description="动作缩放",min=0)
    auto_action_scale:BoolProperty(default=True,description="自动动作缩放")
    subdivide:IntProperty(default=0,description="细分级别",min=0,max=5)
    auto_select_mesh:BoolProperty(default=True,description="自动选择模型")
    auto_select_rigid_body:BoolProperty(default=True,description="自动选择刚体")
    extend_ribbon:BoolProperty(default=True,description="延展飘带区域")
    debug:BoolProperty(default=False,description="debug")
    preset_name:EnumProperty(
        items=get_preset_item,
        description=('Choose the bone type you want to use'),
        default = None,
    )
    IKFK_list=[
            ('unified','unified',''),
            ('IK','IK',''),
            ('FK',"FK",''),
        ]
    IKFK_arm:EnumProperty(
        items=IKFK_list,
        description=('retarget mod'),
        default = 2,
    )
    IKFK_leg:EnumProperty(
        items=IKFK_list,
        description=('retarget mod'),
        default = 2,
    )
    cloth_convert_mod:EnumProperty(
        items=[
            ('Auto','Auto',''),
            ('Bone Constrain','Bone Constrain',''),
            ('Surface Deform','Surface Deform','')
        ],
        description=('retarget mod'),
        default = 0,
    )

class Mmr_Panel_Base(bpy.types.Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "MMR"
    bl_context = "objectmode"
    #bl_options = {'DEFAULT_CLOSED'} # HIDE_HEADER 为隐藏菜单

class MikuMikuRig_1(Mmr_Panel_Base):
    bl_idname="MMR_PT_panel_1"
    bl_label = "Auto MMD rig" #菜单名字

    def draw(self, context):
        prefs = context.preferences
        view= prefs.view
        scene=context.scene
        mmr_property=scene.mmr_property
        layout = self.layout
        layout.label(text="Select MMD armature then press the button")
        layout.operator("mmr.generate_rig",text="Generate MMD rig",icon="CUBE")
        layout.prop(mmr_property,'wrist_rotation_follow',text="Wrist rotation follow arm")
        layout.prop(mmr_property,'auto_shoulder',text="Shoulder IK")
        layout.prop(mmr_property,'solid_rig',text="Replace the controller")
        layout.prop(mmr_property,'pole_target',text="Use pole target")

class MikuMikuRig_2(Mmr_Panel_Base):
    bl_idname="MMR_PT_panel_2"
    bl_label = "Fixed MMD bone dislocation" #菜单名字

    def draw(self, context):
        prefs = context.preferences
        view= prefs.view
        scene=context.scene
        mmr_property=scene.mmr_property
        layout = self.layout
        layout.prop(mmr_property,'min_ik_loop',text="Min IK loop")
        layout.operator("mmr.set_min_ik_loop",text="Set min IK loop",icon="CUBE")

class MikuMikuRig_3(Mmr_Panel_Base):
    bl_idname="MMR_PT_panel_3"
    bl_label = "Auto animation import" #菜单名

    def draw(self, context):
        scene=context.scene
        mmr_property=scene.mmr_property
        layout = self.layout
        layout.label(text="Select rigify controller then press the button")
        layout.prop(mmr_property,'fade_in_out',text="Fade in out")
        layout.prop(mmr_property,'action_scale',text="Animation scale")
        layout.prop(mmr_property,'auto_action_scale',text="Auto mixamo animation scale")
        layout.prop(mmr_property,'lock_location',text="Lock mixamo animation location")
        row=layout.row()
        row.label(text='Arm Type')
        row.prop(mmr_property, 'IKFK_arm', text='')
        row=layout.row()
        row.label(text='Leg Type')
        row.prop(mmr_property, 'IKFK_leg', text='')
        layout.operator("mmr.import_mixamo",text="Import mixamo animation as NLA",icon="CUBE")
        layout.operator("mmr.import_vmd",text="Import VMD animation as NLA",icon="CUBE")
        layout.operator("mmr.export_vmd",text="Bake and export VMD animation",icon="CUBE")

class MikuMikuRig_4(Mmr_Panel_Base):
    bl_idname="MMR_PT_panel_4"
    bl_label = "Auto cloth(experimental)" #菜单名字

    def draw(self, context):
        scene=context.scene
        mmr_property=scene.mmr_property
        layout = self.layout
        layout.label(text="Select mesh and rigidbody then press the button")
        layout.prop(mmr_property,'subdivide',text="Subdivide level")
        row=layout.row()
        row.label(text='Convert Mod')
        row.prop(mmr_property,'cloth_convert_mod',text="")
        layout.prop(mmr_property,'auto_select_mesh',text="Auto select mesh")
        layout.prop(mmr_property,'auto_select_rigid_body',text="Auto select rigid body")
        layout.prop(mmr_property,'extend_ribbon',text="Extend ribbon area")
        layout.operator("mmr.convert_rigid_body_to_cloth",text="Convert rigid body to cloth",icon="CUBE")
        layout.label(text="This featur is developed in cooperation with")
        layout.label(text="UuuNyaa")

class MikuMikuRig_5(Mmr_Panel_Base):
    bl_idname="MMR_PT_panel_5"
    bl_label = "About" #菜单名字
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        scene=context.scene
        mmr_property=scene.mmr_property
        layout = self.layout
        layout.label(text="MikuMikuRig")
        layout.label(text="版本号:"+str(bl_info["version"]))
        layout.label(text="作者:小威廉伯爵")
        layout.prop(mmr_property,'debug',text="Debug")

def alert_error(title,message):
    def draw(self,context):
        self.layout.label(text=str(message))
    bpy.context.window_manager.popup_menu(draw,title=title,icon='ERROR')

class_list=[MikuMikuRig_1,MikuMikuRig_2,MikuMikuRig_3,MikuMikuRig_4,MikuMikuRig_5]
Model_list=[operators]
def register(): #启用插件时候执行
    bpy.utils.register_class(MMR_property)
    bpy.types.Scene.mmr_property = bpy.props.PointerProperty(type=MMR_property)
    for Model in Model_list:
        Model.register()
    for Class in class_list:
        bpy.utils.register_class(Class)
        
    translation.register_module()

    print('register')

def unregister(): #关闭插件时候执行
    for Class in class_list:
        bpy.utils.unregister_class(Class)
    for Model in Model_list:
        Model.unregister()
    bpy.utils.unregister_class(MMR_property)

    translation.unregister_module()

    print('unregister')
